@@ -11,7 +11,6 @@ from time import time
 from webhelpers.feedgenerator import Rss201rev2Feed
 
 from pyramid.response import Response
-from pyramid.i18n import TranslationString as _
 from pyramid.view import view_config
 from pyramid.url import route_url
 from pyramid.httpexceptions import HTTPBadRequest, HTTPFound, HTTPNotFound, HTTPServerError
@@ -25,12 +24,36 @@ from pyrone.models.user import normalize_email
 
 log = logging.getLogger(__name__)
 
+def favicon(fn_key, content_type, request):
+    s = request.registry.settings
+    if fn_key not in s:
+        return HTTPNotFound()
+
+    filename = s[fn_key]
+
+    try:
+        icon = open(filename, 'r')
+    except IOError:
+        return HTTPNotFound()
+
+    return Response(content_type=content_type, app_iter=icon)
+    
+@view_config(route_name='static_favicon_ico')
+def favicon_ico(request):
+    return favicon('pyrone.favicon_ico', 'image/x-icon', request)
+
+@view_config(route_name='static_favicon_png')
+def favicon_png(request):
+    return favicon('pyrone.favicon_png', 'image/png', request)
+
+
 @view_config(route_name='blog_latest', renderer='/blog/list_articles.mako')
 def latest(request):
     """
     Display list of articles sorted by publishing date ascending,
     show rendered previews, not complete articles
     """
+    _ = request.translate
     c = dict(articles=list())
     
     h = []
@@ -72,6 +95,7 @@ def latest(request):
 
 @view_config(route_name='blog_tag_articles', renderer='/blog/list_articles.mako')
 def tag_articles(request):
+    _ = request.translate
     tag = request.matchdict['tag']
     
     page_size = int(get_config('elements_on_page'))
@@ -112,6 +136,7 @@ def _check_article_fields(article, request):
     """
     Read article from the POST request, also validate data
     """
+    _ = request.translate
     errors = dict()
     # check passed data
     fields = ['title', 'shortcut', 'body']
@@ -137,8 +162,76 @@ def _check_article_fields(article, request):
     
     return errors
 
+def _update_article(article_id, request):
+    _ = request.translate
+
+    dbsession = DBSession()
+
+    transaction.begin()
+    article = dbsession.query(Article).get(article_id)
+    if article is None:
+        return HTTPNotFound()
+
+    # check fields etc
+    e = _check_article_fields(article, request)
+    c = dict(errors={})
+    c['article'] = article
+    c['errors'].update(e)
+    c['article_published_str'] = request.POST['published']
+    
+    if 'published' not in request.POST:
+        c['errors']['published'] = _('invalid date and time format')
+    else:
+        # parse value to check structure
+        date_re = re.compile('^([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2}):([0-9]{2})$')
+        mo = date_re.match(request.POST['published'])
+        if mo is None:
+            c['errors']['published'] = _('invalid date and time format')
+        else:
+            # we need to convert LOCAL date and time to UTC seconds
+            article.published = h.str_to_timestamp(request.POST['published'])
+            article.shortcut_date = '%04d/%02d/%02d' % tuple([int(x) for x in mo.groups()[0:3]])
+        
+        dbsession = DBSession()
+        q = dbsession.query(Article).filter(Article.shortcut_date==article.shortcut_date)\
+            .filter(Article.id != article_id)\
+            .filter(Article.shortcut==article.shortcut)
+        res = q.first()
+
+        if res is not None:
+            c['errors']['shortcut'] = _('duplicated shortcut')
+            
+    # tags
+    c['tags'] = [] # these are new tags
+    if 'tags' in request.POST:
+        tags_str = request.POST['tags']
+        tags = set([s.strip() for s in tags_str.split(',')])
+
+        for tag_str in tags:
+            if tag_str == '':
+                continue
+            c['tags'].append(tag_str)
+            
+    if len(c['errors']) == 0:
+        for tag in article.tags:
+            dbsession.delete(tag)
+            
+        for tag_str in c['tags']:
+            tag = Tag(tag_str, article)
+            dbsession.add(tag)
+            
+        transaction.commit()
+        
+        return HTTPFound(location=route_url('blog_go_article', request, article_id=article_id))
+    else:
+        transaction.abort()
+
+    return c
+
+
 @view_config(route_name='blog_write_article', renderer='/blog/write_article.mako', permission='write_article')
 def write_article(request):
+    _ = request.translate
     c = dict(
          new_article=True, 
          submit_url=route_url('blog_write_article', request),
@@ -216,6 +309,25 @@ def write_article(request):
         
     return c
 
+@view_config(route_name='blog_edit_article_ajax', permission='write_article', renderer='json', request_method='POST')
+def edit_article_ajax(request):
+    article_id = int(request.matchdict['article_id'])
+    log.debug(request.translate('field is required'))
+
+    res = _update_article(article_id, request)
+    
+    if isinstance(res, HTTPFound):
+        return dict()
+
+    if type(res) != dict:
+        return res
+
+    c = dict()
+    if len(res['errors']) != 0:
+        c['errors'] = res['errors']
+
+    return c
+
 @view_config(route_name='blog_edit_article', renderer='/blog/write_article.mako', permission='write_article')
 def edit_article(request):
     article_id = int(request.matchdict['article_id'])
@@ -227,71 +339,18 @@ def edit_article(request):
     
     if request.method == 'GET':
         article = dbsession.query(Article).get(article_id)
+        c['article'] = article
         c['tags'] = [tag.tag for tag in article.tags]
         c['article_published_str'] = h.timestamp_to_str(article.published)
     elif request.method == 'POST':
-        transaction.begin()
-        article = dbsession.query(Article).get(article_id)
-        # check fields etc
-        e = _check_article_fields(article, request)
-        c['errors'].update(e)
-        c['article_published_str'] = request.POST['published']
-        
-        if 'published' not in request.POST:
-            c['errors']['published'] = _('invalid date and time format')
-        else:
-            # parse value to check structure
-            date_re = re.compile('^([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2}):([0-9]{2})$')
-            mo = date_re.match(request.POST['published'])
-            if mo is None:
-                c['errors']['published'] = _('invalid date and time format')
-            else:
-                # we need to convert LOCAL date and time to UTC seconds
-                article.published = h.str_to_timestamp(request.POST['published'])
-                article.shortcut_date = '%04d/%02d/%02d' % tuple([int(x) for x in mo.groups()[0:3]])
-            
-            dbsession = DBSession()
-            q = dbsession.query(Article).filter(Article.shortcut_date==article.shortcut_date)\
-                .filter(Article.id != article_id)\
-                .filter(Article.shortcut==article.shortcut)
-            res = q.first()
-    
-            if res is not None:
-                c['errors']['shortcut'] = _('duplicated shortcut')
-                
-        # tags
-        c['tags'] = [] # these are new tags
-        if 'tags' in request.POST:
-            tags_str = request.POST['tags']
-            tags = set([s.strip() for s in tags_str.split(',')])
+        res = _update_article(article_id, request)
+        if type(res) != dict:
+            return res
 
-            for tag_str in tags:
-                if tag_str == '':
-                    continue
-                c['tags'].append(tag_str)
-                
-        if len(c['errors']) == 0:
-            for tag in article.tags:
-                dbsession.delete(tag)
-                
-            for tag_str in c['tags']:
-                tag = Tag(tag_str, article)
-                dbsession.add(tag)
-                
-            transaction.commit()
-            
-            return HTTPFound(location=route_url('blog_go_article', request, article_id=article_id))
-        else:
-            transaction.abort()
-        
-    else:
-        return HTTPBadRequest()
+        c.update(res)
     
-    if article is None:
-        return HTTPNotFound()
-    
-    c['article'] = article
     c['submit_url'] = route_url('blog_edit_article', request, article_id=article_id)
+    c['save_url_ajax'] = route_url('blog_edit_article_ajax', request, article_id=article_id)
     return c
 
 @view_config(route_name='blog_article_delete_ajax', renderer='json', permission='admin', request_method='POST')
@@ -331,6 +390,10 @@ def _view_article(request, article_id=None, article=None):
         
     if article is None:
         return HTTPNotFound()
+
+    user = auth.get_user(request)
+    if article.is_draft and not user.has_permission('edit_article'):
+        return HTTPNotFound()
     
     comments = dbsession.query(Comment).filter(Comment.article==article).all()
     comments_dict = dict()
@@ -406,6 +469,7 @@ def view_article(request):
 
 @view_config(route_name='blog_add_article_comment_ajax', renderer='json', request_method='POST')
 def add_article_comment_ajax(request):
+    _ = request.translate
     article_id = int(request.matchdict['article_id'])
     
     dbsession = DBSession()
@@ -832,6 +896,7 @@ def latest_rss(request):
     """
     Create rss feed with the latest published articles and return them as the atom feed
     """
+    _ = request.translate
     dbsession= DBSession()
     
     q = dbsession.query(Article).options(eagerload('tags')).options(eagerload('user')).filter(Article.is_draft==False).order_by(Article.updated.desc())
